@@ -371,7 +371,8 @@ class DefDirectiveTestCase(unittest.TestCase):
           #end
           ${echo('Hi', name='you')}
         """)
-        self.assertEqual("""                      Hi, you!
+        self.assertEqual("""
+                      Hi, you!
         """, str(tmpl.generate()))
 
 
@@ -590,6 +591,54 @@ class MatchDirectiveTestCase(unittest.TestCase):
           </body>
         </html>""", str(tmpl.generate()))
 
+    def test_not_match_self(self):
+        """
+        See http://genshi.edgewall.org/ticket/77
+        """
+        tmpl = MarkupTemplate("""<html xmlns="http://www.w3.org/1999/xhtml"
+              xmlns:py="http://genshi.edgewall.org/">
+          <body py:match="body" py:content="select('*')" />
+          <h1 py:match="h1">
+            ${select('text()')}
+            Goodbye!
+          </h1>
+          <body>
+            <h1>Hello!</h1>
+          </body>
+        </html>""")
+        self.assertEqual("""<html xmlns="http://www.w3.org/1999/xhtml">
+          <body><h1>
+            Hello!
+            Goodbye!
+          </h1></body>
+        </html>""", str(tmpl.generate()))
+
+    def test_select_text_in_element(self):
+        """
+        See http://genshi.edgewall.org/ticket/77#comment:1
+        """
+        tmpl = MarkupTemplate("""<html xmlns="http://www.w3.org/1999/xhtml"
+              xmlns:py="http://genshi.edgewall.org/">
+          <body py:match="body" py:content="select('*')" />
+          <h1 py:match="h1">
+            <text>
+              ${select('text()')}
+            </text>
+            Goodbye!
+          </h1>
+          <body>
+            <h1>Hello!</h1>
+          </body>
+        </html>""")
+        self.assertEqual("""<html xmlns="http://www.w3.org/1999/xhtml">
+          <body><h1>
+            <text>
+              Hello!
+            </text>
+            Goodbye!
+          </h1></body>
+        </html>""", str(tmpl.generate()))
+
     def test_select_all_attrs(self):
         tmpl = MarkupTemplate("""<doc xmlns:py="http://genshi.edgewall.org/">
           <div py:match="elem" py:attrs="select('@*')">
@@ -719,6 +768,21 @@ class MatchDirectiveTestCase(unittest.TestCase):
           <div>Foo</div>
         </html>""", str(tmpl.generate()))
 
+    def test_match_without_closure(self):
+        tmpl = MarkupTemplate("""<html xmlns:py="http://genshi.edgewall.org/">
+          <p py:match="body/p" class="para">${select('*|text()')}</p>
+          <body>
+            <p>Foo</p>
+            <div><p>Bar</p></div>
+          </body>
+        </html>""")
+        self.assertEqual("""<html>
+          <body>
+            <p class="para">Foo</p>
+            <div><p>Bar</p></div>
+          </body>
+        </html>""", str(tmpl.generate()))
+
     # FIXME
     #def test_match_after_step(self):
     #    tmpl = MarkupTemplate("""<div xmlns:py="http://genshi.edgewall.org/">
@@ -1105,13 +1169,44 @@ class TextTemplateTestCase(unittest.TestCase):
         #if foo
           bar
         #end 'if foo'""")
-        self.assertEqual('', str(tmpl.generate()))
+        self.assertEqual('\n', str(tmpl.generate()))
 
     def test_latin1_encoded(self):
         text = u'$foo\xf6$bar'.encode('iso-8859-1')
         tmpl = TextTemplate(text, encoding='iso-8859-1')
         self.assertEqual(u'x\xf6y', unicode(tmpl.generate(foo='x', bar='y')))
 
+    def test_empty_lines1(self):
+        tmpl = TextTemplate("""Your items:
+
+        #for item in items
+          * ${item}
+        #end""")
+        self.assertEqual("""Your items:
+
+          * 0
+          * 1
+          * 2
+""", tmpl.generate(items=range(3)).render('text'))
+
+    def test_empty_lines2(self):
+        tmpl = TextTemplate("""Your items:
+
+        #for item in items
+          * ${item}
+
+        #end""")
+        self.assertEqual("""Your items:
+
+          * 0
+
+          * 1
+
+          * 2
+
+""", tmpl.generate(items=range(3)).render('text'))
+
+
 class TemplateLoaderTestCase(unittest.TestCase):
     """Tests for the template loader."""
 
